#!/bin/sh

# ============================================================
# INSTALLATION SCRIPT FOR PRAGMADEV REAL TIME DEVELOPER STUDIO
# ============================================================
# Copyright (c) PragmaDev 2002
#
# History:
# ------------------------------------------------------------
# Version | Comment
# ------------------------------------------------------------
# 1.0     | Creation
# 1.1     | Exclude share/3rdparty/MinGW when installing on
#         | and for Unix
# 1.2     | Clean-up target directory before installing
# ------------------------------------------------------------

## Function reading a line from the terminal
getAnswer()
{
  ## Read from terminal
  read answer
  ## Quit if needed
  if [ "$answer" = "q" ]
  then
    exit 0
  fi
  ## Output answer
  echo $answer
}

## Welcome screen
cat <<.
=======================================
REAL TIME DEVELOPER STUDIO INSTALLATION
=======================================
This script will install PragmaDev Real Time Developer Studio.

Press return to continue or 'q' to quit.
.
getAnswer > /dev/null

## Architecture may be passed as an argument
arch=$1

## If it wasn't, try to figure it out
if [ "$arch" = "" ]
then
  case `uname -s` in
    Linux)
      arch='linux'
      ;;
    SunOS)
      case `uname -r` in
        5.10)
          arch='solaris10intel'
          ;;
        *)
          arch='solaris8sparc'
          ;;
        esac
      ;;
  esac
fi

## If architecture still not found or invalid, can't proceed
if [ "$arch" != "linux" -a "$arch" != "solaris8sparc" -a "$arch" != "solaris10intel" ]
then
  echo '!!! ERROR !!!'
  echo 'Unable to determine architecture.'
  echo 'You may try to re-run this installation script passing the architecture'
  echo 'as an argument. Valid architectures are "linux", "solaris8sparc" and'
  echo 'solaris10intel.'
  exit 1
fi

## Directory for install script is the mount point of the CD-ROM
rootDir=`dirname $0`

## Display license agreement
more $rootDir/doc/LICENSE.txt
echo ""
echo "Answer 'yes' to accept this license agreement. Any other answer will quit the installation."
license_answer=`getAnswer`
if [ "$license_answer" != "yes" ]
then
  echo "Licence refused. Quitting..."
  exit 0
fi

## Ask for installation directory
installDir=""
while [ "$installDir" = "" ]
do
  echo ""
  echo "Please enter the installation directory for Real Time Developer Studio"
  echo "(default: /opt/pragmadev/rtds; 'q' to quit)"
  installDir=`getAnswer`
  if [ "$installDir" = "" ]
  then
    installDir='/opt/pragmadev/rtds'
  fi

  ## If installation directory already exists
  if [ -r "$installDir" ]
  then
    ## If it's not a directory, error...
    if [ ! -d "$installDir" ]
    then
      echo '!!! ERROR !!!'
      echo "File $installDir exists, but is not a directory. Please choose another"
      echo "location."
      echo ""
      echo "Press return to continue or 'q' to quit"
      getAnswer > /dev/null
      installDir=""
    else
      ## If directory is not writable, error
      if [ ! -w "$installDir" ]
      then
        echo '!!! ERROR !!!'
        echo "Directory $installDir exists, but is not writable. Please choose another"
        echo "location, or quit the installation and re-log as a user with sufficient"
        echo "rights to write in this directory"
        echo ""
        echo "Press return to continue or 'q' to quit"
        getAnswer > /dev/null
        installDir=""
      ## If directory is writable
      else
        ## Directory will be emptied; ask confirmation
        echo "--- WARNING! ---"
        echo "Existing directory $installDir will be emptied before installation!"
        echo "Do you really want to install in $installDir ([y]/n/q)?"
        answer=`getAnswer`
        if [ "$answer" = "n" ]
        then
          installDir=""
        fi
      fi
    fi
  ## If installation directory does not exist, ask if we should create it
  else
    echo "Directory $installDir does not exist. Do you want to create it ([y]/n/q)?"
    answer=`getAnswer`
    if [ "$answer" != "n" ]
    then
      mkdir -p $installDir
      ## If creation failed
      if [ $? -ne 0 ]
      then
        echo '!!! ERROR !!!'
        echo "Could not create installation directory $installDir"
        echo "Make sure you have the appropriate rights or choose another directory"
        echo ""
        echo "Press return to continue or enter 'q' to quit"
        getAnswer > /dev/null
        installDir=""
      fi
    else
      installDir=""
    fi
  fi
done

## Confirmation screen
cat <<.

Real Time Developer Studio will now be installed with the following parameters:

 - Architecture: $arch
 
 - Installation directory: $installDir

PLEASE CHECK THE INFORMATIONS ABOVE. If you press return now, Real Time Developer
Studio will be installed. If any of the informations above is not correct, enter
'q' and re-run the installation script from the beginning.
.
getAnswer > /dev/null

## Clean-up destination directory
echo '--- Cleaning up installation directory'
( cd $installDir ; rm -rf `ls -A` )

## Install binaries
echo '--- Installing RTDS binaries'
cd $rootDir
if [ ! -r $installDir/bin ]
then
  mkdir $installDir/bin
fi
cp -r bin/$arch/* $installDir/bin/

## Install documentation
echo '--- Installing RTDS documentation'
tar cf - doc | (cd $installDir ; tar xf -)

## Install common files
## NB: the --exclude tar option is a GNU tar extension; don't try to use it on platforms
## that do not have GNU tar installed by default (namely Solaris)
echo '--- Installing RTDS common files'
if [ "$arch" = "windows" -o "`uname -s`" = "SunOS" ]
then
  tarOptions=''
else
  tarOptions='--exclude=share/3rdparty/MinGW'
fi
tar cf - $tarOptions share | (cd $installDir ; tar xf -)
## Change permissions on share/ccg to make sure files won't be deleted by mistake
chmod -R a-w $installDir/share/ccg

## Install contributed files
if [ -d contrib ]
then
  echo '--- Installing contributed files'
  tar cf - contrib | (cd $installDir ; tar xf -)
fi

## Install examples
echo '--- Installing RTDS example files'
tar cf - examples | (cd $installDir ; tar xf -)

## Creating configuration scripts
echo '--- Creating configuration scripts'
echo "RTDS_HOME=$installDir" > $installDir/rtds-config.sh
echo 'PATH=$RTDS_HOME/bin:"$PATH"' >> $installDir/rtds-config.sh
echo 'LD_LIBRARY_PATH=$RTDS_HOME/bin:"$LD_LIBRARY_PATH"' >> $installDir/rtds-config.sh
echo 'export RTDS_HOME PATH LD_LIBRARY_PATH' >> $installDir/rtds-config.sh

echo "setenv RTDS_HOME $installDir" > $installDir/rtds-config.csh
echo 'setenv PATH $RTDS_HOME/bin:"$PATH"' >> $installDir/rtds-config.csh
echo 'setenv LD_LIBRARY_PATH $RTDS_HOME/bin:"$LD_LIBRARY_PATH"' >> $installDir/rtds-config.csh


## Done; display post-installation readme file
cat - $rootDir/doc/INSTALL.txt <<. | more
--- Installation complete.

.
echo 'Press return'
read dummy

## Over
echo ''
echo 'Installation completed successfully!'
echo ''
echo "WARNING: to be able to run Real Time Developer Studio, each account must have defined the"
echo "environment variable RTDS_HOME to $installDir, and included the path to RTDS binaries in"
echo "its PATH and LD_LIBRARY_PATH environment variables"
echo ""
echo "To do this:"
echo ""
echo " - on sh compatible shells (sh, ksh, bash), source in your .profile the script"
echo "   $installDir/rtds-config.sh"
echo ""
echo " - on csh compatible shells (csh, tcsh), source in your .cshrc the script"
echo "   $installDir/rtds-config.csh"
echo ''
echo 'Welcome to PragmaDev Real Time Developer Studio!'

exit 0
