/* PragmaDev RTDS FreeRTOS integration */
/* Integration with FreeRTOS RTOS      */

#ifndef _RTDS_MACRO_H_
#define _RTDS_MACRO_H_

#ifdef __cplusplus
extern "C" {
#endif

/* Standard includes */
#include "stdio.h"

/* RTDS includes */
#include "RTDS_BasicTypes.h"
#include "RTDS_OS.h"
#include "RTDS_Error.h"
#ifdef RTDS_BACK_TRACE_MAX_EVENT_NUM
#include "RTDS_BackTrace.h"
#endif

/* TCP Client include*/
#include "RTDS_TCP_Client.h"

/* ************************************************************************************ *
 *									TASK ENTRY POINT DEFINITION AND PROTOTYPE
 * ************************************************************************************ */
#define RTDS_TASK_ENTRY_POINT(TASK_NAME) \
	void TASK_NAME(RTDS_GlobalProcessInfo *RTDS_currentContext)
#define RTDS_TASK_ENTRY_POINT_PROTO(TASK_NAME) extern RTDS_TASK_ENTRY_POINT(TASK_NAME)


#ifdef RTDS_FREERTOS_WINDOWS_SIMULATOR
    /* ************************************************************************************ *
     *                             CRITICAL TRACE SECTION MACROS
     * ************************************************************************************ */
    #ifdef RTDS_SIMULATOR
    #define RTDS_CRITICAL_TRACE_SECTION_DECL        RTDS_WindowsSemaphoreId RTDS_globalTraceSemId;
    #define RTDS_CRITICAL_TRACE_SECTION_PROTO       extern RTDS_WindowsSemaphoreId RTDS_globalTraceSemId;
    #define RTDS_CRITICAL_TRACE_SECTION_INIT        RTDS_globalTraceSemId = RTDS_SemMCreate();
    #define RTDS_CRITICAL_TRACE_SECTION_START       RTDS_SemTake( RTDS_globalTraceSemId, RTDS_SEMAPHORE_TIME_OUT_FOREVER );
    #define RTDS_CRITICAL_TRACE_SECTION_STOP        RTDS_SemGive( RTDS_globalTraceSemId );
    #define RTDS_CRITICAL_TRACE_SECTION_POSTAMBLE   RTDS_SemDelete( RTDS_globalTraceSemId );
    #else
    #define RTDS_CRITICAL_TRACE_SECTION_DECL
    #define RTDS_CRITICAL_TRACE_SECTION_PROTO
    #define RTDS_CRITICAL_TRACE_SECTION_INIT
    #define RTDS_CRITICAL_TRACE_SECTION_START
    #define RTDS_CRITICAL_TRACE_SECTION_STOP
    #define RTDS_CRITICAL_TRACE_SECTION_POSTAMBLE
    #endif


    /* ************************************************************************************ *
     *                                SOCKET SYNCHRONISATION
     * Defines when tracing through socket to synchronize target execution and host debugger
     * ************************************************************************************ */
    #define RTDS_DTRACE_ACK_NOWAIT          0     /* No acknowledgment from the host */
    #define RTDS_DTRACE_ACK_WAIT            1     /* Ack from the host */

    #if ( defined( RTDS_SIMULATOR ) || defined( RTDS_MSC_TRACER ) ) && defined( RTDS_SOCKET_PORT )
     #define RTDS_SOCKET_ACCESS_DECL        RTDS_SemaphoreId RTDS_globalSocketAccess;
     #define RTDS_SOCKET_ACCESS_PROTO       extern RTDS_SemaphoreId RTDS_globalSocketAccess;
     #define RTDS_SOCKET_ACCESS_INIT        RTDS_globalSocketAccess = RTDS_SemMCreate();
     #define RTDS_SOCKET_ACCESS_TAKE        RTDS_SemTake( RTDS_globalSocketAccess, RTDS_SEMAPHORE_TIME_OUT_FOREVER );
     #define RTDS_SOCKET_ACCESS_GIVE        RTDS_SemGive( RTDS_globalSocketAccess );
    #else
     #define RTDS_SOCKET_ACCESS_DECL
     #define RTDS_SOCKET_ACCESS_PROTO
     #define RTDS_SOCKET_ACCESS_INIT
     #define RTDS_SOCKET_ACCESS_TAKE
     #define RTDS_SOCKET_ACCESS_GIVE
    #endif

    #if defined( RTDS_SOCKET_PORT ) && defined( RTDS_SIMULATOR )
     #define RTDS_DTRACE_ACKNOWLEDGE_DECL       RTDS_SemaphoreId RTDS_globalTcpEventSynchro;
     #define RTDS_DTRACE_ACKNOWLEDGE_PROTO      extern RTDS_SemaphoreId RTDS_globalTcpEventSynchro;
     #define RTDS_DTRACE_ACKNOWLEDGE_INIT       RTDS_globalTcpEventSynchro = RTDS_SemBCreate( 0 );
     #define RTDS_DTRACE_ACKNOWLEDGE_WAIT       RTDS_SemTake( RTDS_globalTcpEventSynchro, RTDS_SEMAPHORE_TIME_OUT_FOREVER );
     #define RTDS_DTRACE_ACKNOWLEDGE_RECEIVED   RTDS_SemGive( RTDS_globalTcpEventSynchro );
    #else
     #define RTDS_DTRACE_ACKNOWLEDGE_DECL
     #define RTDS_DTRACE_ACKNOWLEDGE_PROTO
     #define RTDS_DTRACE_ACKNOWLEDGE_INIT
     #define RTDS_DTRACE_ACKNOWLEDGE_WAIT
     #define RTDS_DTRACE_ACKNOWLEDGE_RECEIVED
    #endif
#endif

/* ************************************************************************************ *
 *                                        SYSTEM MACRO
 * ************************************************************************************ */
/*
 * MACRO RTDS_SYSTEM_ERROR:
 * ------------------------
 * Macro called when a system error is detected in RTDS 'kernel'
 * Needs to be defined by user.
 */
#ifndef RTDS_SYSTEM_ERROR
    #define RTDS_SYSTEM_ERROR(errorNumber) \
    	{ \
    	RTDS_SIMULATOR_TRACE(RTDS_systemError, errorNumber, NULL, NULL); \
    	printf("RTDS error 0x%X\nCheck RTDS_Error.h file for explanations.\n",errorNumber); \
    	exit(errorNumber); \
    	}
#endif

/*
 * SIMULATOR TRACE MACRO:
 * ------------------------
 * Macro called to trace an SDL-RT event.
 */
#if defined( RTDS_SIMULATOR ) || defined( RTDS_MSC_TRACER )
    #define RTDS_SIMULATOR_TRACE( EVENT, PARAM1, PARAM2, RTDS_CURRENT_CONTEXT ) \
        RTDS_SimulatorTrace( ( enum RTDS_EventType )EVENT, ( void * )PARAM1, ( long )PARAM2, ( RTDS_GlobalProcessInfo * )RTDS_CURRENT_CONTEXT, RTDS_DTRACE_ACK_WAIT );
#else
    #if defined(RTDS_SIMULATOR) || defined(RTDS_BACK_TRACE_MAX_EVENT_NUM)
        #ifdef RTDS_BACK_TRACE_MAX_EVENT_NUM
            #define RTDS_SIMULATOR_TRACE(EVENT, PARAM1, PARAM2, PARAM3) \
                RTDS_globalTraceEntry.event = EVENT; \
                RTDS_globalTraceEntry.eventParameter1 = (void *) PARAM1; \
                RTDS_globalTraceEntry.eventParameter2 = (long) PARAM2; \
                RTDS_globalTraceEntry.currentContext = (RTDS_GlobalProcessInfo *) PARAM3; \
                RTDS_TraceAdd(); \
                RTDS_DummyTraceFunction();
        #else
            #define RTDS_SIMULATOR_TRACE(EVENT, PARAM1, PARAM2, PARAM3) \
                RTDS_globalTraceEntry.event = EVENT; \
                RTDS_globalTraceEntry.eventParameter1 = (void *) PARAM1; \
                RTDS_globalTraceEntry.eventParameter2 = (long) PARAM2; \
                RTDS_globalTraceEntry.currentContext = (RTDS_GlobalProcessInfo *) PARAM3; \
                printf("RTDS event: %d\n", EVENT); \
                RTDS_DummyTraceFunction();
        #endif
    #else
        #define RTDS_SIMULATOR_TRACE(EVENT, PARAM1, PARAM2, PARAM3)
    #endif
#endif





/* ************************************************************************************ *
 *                              SDL KEYWORD DEFINITION
 *			Define the SDL PARENT, SELF, OFFSPRING, and SENDER keywords.
 * ************************************************************************************ */

#define PARENT     RTDS_currentContext->parentSdlInstanceId
#define SELF       RTDS_currentContext->mySdlInstanceId
#define OFFSPRING  RTDS_currentContext->offspringSdlInstanceId
#define SENDER     RTDS_senderQueueId


/* ************************************************************************************ *
 *                                   MEMORY MACROS
 * ************************************************************************************ */

/*
 * MACRO RTDS_MALLOC:
 * ---------------------
 * Memory allocation
 */

#define RTDS_MALLOC(SIZE)  pvPortMalloc(SIZE)


/*
 * MACRO RTDS_FREE:
 * -------------------
 * Memory liberation
 */

#define RTDS_FREE(PTR)	vPortFree(PTR)



/* ************************************************************************************ *
 *                                   TIMER MACROS
 * ************************************************************************************ */

/*
 * MACRO RTDS_RESET_TIMER:
 * --------------------------
 * Cancels a timer
 */

#define RTDS_RESET_TIMER(TIMER_NUMBER) \
		RTDS_StopTimer(TIMER_NUMBER, &(RTDS_currentContext->timerList), RTDS_currentContext);


/*
 * MACRO RTDS_SET_TIMER:
 * ------------------------
 * Initializes and starts a timer
 */

#define RTDS_SET_TIMER(TIMER_NUMBER, DELAY) \
	RTDS_RESET_TIMER(TIMER_NUMBER); \
	RTDS_StartTimer(RTDS_currentContext->mySdlInstanceId, TIMER_NUMBER, RTDS_GetTimerUniqueId(RTDS_currentContext->timerList),DELAY, &(RTDS_currentContext->timerList), RTDS_currentContext);



/* ************************************************************************************ *
 *                                     MESSAGE MACROS
 * ************************************************************************************ */

/*
 * MACRO RTDS_MSG_QUEUE_READ:
 * -----------------------------
 * Reads the next message in the process's message queue
 * Memory needs to be allocated before calling the macro
 */

#define RTDS_MSG_QUEUE_READ(CURRENT_MESSAGE) \
		RTDS_MsgQueueReceive((RTDS_SdlInstanceId *)(RTDS_currentContext->mySdlInstanceId), (unsigned char *)CURRENT_MESSAGE, RTDS_WAIT_FOREVER);


/*
 * MACRO RTDS_MSG_QUEUE_SEND_TO_ID:
 * ----------------------------------
 * Sends a message to a process's message queue using its queue id.
 */

#define RTDS_MSG_QUEUE_SEND_TO_ID(MESSAGE_NUMBER, LENGTH_DATA, P_DATA, RECEIVER) \
		RTDS_MsgQueueSend(MESSAGE_NUMBER, LENGTH_DATA, (unsigned char *) P_DATA, (RTDS_SdlInstanceId *) RECEIVER, SELF, RTDS_currentContext);


/*
 * MACRO RTDS_MSG_QUEUE_SEND_TO_ENV:
 * ------------------------------------
 * Sends a message to the environment.
 */

#define RTDS_MSG_QUEUE_SEND_TO_ENV(MESSAGE_NUMBER, LENGTH_DATA, P_DATA) \
		RTDS_MsgQueueSend(MESSAGE_NUMBER, LENGTH_DATA, (unsigned char *) P_DATA, RTDS_GetProcessQueueId(RTDS_process_RTDS_Env), SELF, RTDS_currentContext);


/*
 * MACRO RTDS_MSG_QUEUE_SEND_TO_NAME:
 * ------------------------------------
 * Sends a message to a process's message queue using its name.
 * The receiver name is provided as a string and as a number defined in RTDS_gen.h
 */

#define RTDS_MSG_QUEUE_SEND_TO_NAME(MESSAGE_NUMBER, LENGTH_DATA, P_DATA, RECEIVER_STRING, RECEIVER_NUMBER) \
		RTDS_MsgQueueSend(MESSAGE_NUMBER, LENGTH_DATA, (unsigned char *) P_DATA, RTDS_GetProcessQueueId(RECEIVER_NUMBER), SELF, RTDS_currentContext);


/*
 * MACRO RTDS_MSG_SAVE:
 * -----------------------
 * Saves a message to the save queue chained list
 */

#define RTDS_MSG_SAVE(CURRENT_MESSAGE) \
	RTDS_SIMULATOR_TRACE(RTDS_messageSaved, CURRENT_MESSAGE, SELF, RTDS_currentContext); \
	CURRENT_MESSAGE->next=NULL; \
	if ( RTDS_currentContext->writeSaveQueue == NULL ) \
		RTDS_currentContext->writeSaveQueue = CURRENT_MESSAGE; \
	else \
		{ \
		RTDS_MessageHeader	*RTDS_MSG_SAVE_message; \
		for (RTDS_MSG_SAVE_message=RTDS_currentContext->writeSaveQueue ; RTDS_MSG_SAVE_message->next != NULL ; RTDS_MSG_SAVE_message = RTDS_MSG_SAVE_message->next); \
		RTDS_MSG_SAVE_message->next = CURRENT_MESSAGE; \
		} \
	CURRENT_MESSAGE = NULL;



/*
 * MACRO RTDS_MSG_INPUT_ERROR:
 * --------------------------
 * Macro called in the generated code if the pointer on the received parameters is NULL.
 */

#define RTDS_MSG_INPUT_ERROR \
	RTDS_SYSTEM_ERROR(RTDS_ERROR_MSG_INPUT);




/* ************************************************************************************ *
 *                                       SEMAPHORE MACROS
 * ************************************************************************************ */

/*
 * MACRO RTDS_BINARY_SEMAPHORE_CREATE:
 * -------------------------------------
 * Creates a binary semaphore
 */

#define RTDS_BINARY_SEMAPHORE_CREATE(SEMAPHORE_NAME, SEMAPHORE_NUMBER, OPTIONS, INITIAL_STATE) \
	RTDS_BinarySemaphoreCreate(SEMAPHORE_NUMBER, INITIAL_STATE, RTDS_currentContext);


/*
 * MACRO RTDS_MUTEX_SEMAPHORE_CREATE:
 * -------------------------------------
 * Creates a mutex semaphore
 */

#define RTDS_MUTEX_SEMAPHORE_CREATE(SEMAPHORE_NAME, SEMAPHORE_NUMBER, OPTIONS) \
	RTDS_MutexSemaphoreCreate(SEMAPHORE_NUMBER, RTDS_currentContext);


/*
 * MACRO RTDS_COUNTING_SEMAPHORE_CREATE:
 * ---------------------------------------
 * Creates a counting semaphore
 * Posix like
 */

#define RTDS_COUNTING_SEMAPHORE_CREATE(SEMAPHORE_NAME, SEMAPHORE_NUMBER, OPTIONS, INITIAL_COUNT) \
	RTDS_CountingSemaphoreCreate(SEMAPHORE_NUMBER, INITIAL_COUNT, RTDS_currentContext);


/*
 * MACRO RTDS_SEMAPHORE_DELETE:
 * -----------------------------
 * Creates a counting semaphore
 */

#define RTDS_SEMAPHORE_DELETE( SEMAPHORE_ID ) \
		RTDS_Sem_Delete( SEMAPHORE_ID, RTDS_currentContext );


#define RTDS_SEMAPHORE_NAME_DELETE( SEMAPHORE_NAME, SEMAPHORE_NUMBER ) \
		RTDS_Sem_Delete( RTDS_GetSemaphoreId( SEMAPHORE_NUMBER), RTDS_currentContext );

/*
 * MACRO RTDS_SEMAPHORE_TAKE:
 * -----------------------------
 * Takes a semaphore
 */

#define RTDS_SEMAPHORE_ID_TAKE( SEMAPHORE_ID, TIME_OUT) \
		RTDS_SemaphoreIdTake( SEMAPHORE_ID, TIME_OUT, RTDS_currentContext ) ;



#define RTDS_SEMAPHORE_NAME_TAKE( SEMAPHORE_NAME, SEMAPHORE_NUMBER, TIME_OUT ) \
		RTDS_SemaphoreIdTake( RTDS_GetSemaphoreId(SEMAPHORE_NUMBER), TIME_OUT, RTDS_currentContext ) ;


/*
 * MACRO RTDS_SEMAPHORE_GIVE:
 * -----------------------------
 * Gives a semaphore
 */

#define RTDS_SEMAPHORE_ID_GIVE(SEMAPHORE_ID) \
		RTDS_SIMULATOR_TRACE(RTDS_semGive, SEMAPHORE_ID, NULL, RTDS_currentContext); \
		xSemaphoreGive(SEMAPHORE_ID);



#define RTDS_SEMAPHORE_NAME_GIVE(SEMAPHORE_NAME, SEMAPHORE_NUMBER) \
		RTDS_SIMULATOR_TRACE(RTDS_semGive, RTDS_GetSemaphoreId(SEMAPHORE_NUMBER), NULL, RTDS_currentContext); \
		xSemaphoreGive( RTDS_GetSemaphoreId(SEMAPHORE_NUMBER) );



/* ************************************************************************************ *
 *                                      PROCESS MACROS
 * ************************************************************************************ */

/*
 * MACRO RTDS_PROCESS_CREATE:
 * -----------------------------
 * Creates a new task representing an SDL process
 */

#define RTDS_PROCESS_CREATE(PROCESS_NAME, PROCESS_NUMBER, FUNCTION_ADDRESS, PRIORITY) \
		RTDS_ProcessCreate(PROCESS_NAME, PROCESS_NUMBER, FUNCTION_ADDRESS, PRIORITY, &OFFSPRING, SELF, RTDS_HOLD, RTDS_currentContext);


/*
 * MACRO RTDS_STARTUP_PROCESS_CREATE:
 * -----------------------------
 * Used by Startup task to create a new task representing an SDL process
 */

#define RTDS_STARTUP_PROCESS_CREATE(PROCESS_NAME, PROCESS_NUMBER, FUNCTION_ADDRESS, PRIORITY) \
		RTDS_ProcessCreate(PROCESS_NAME, PROCESS_NUMBER, FUNCTION_ADDRESS, PRIORITY, &OFFSPRING, SELF, RTDS_NO_HOLD, RTDS_currentContext);

/*		RTDS_ProcessCreate(FUNCTION_NAME, (void *)FUNCTION_ADDRESS, PRIORITY, &OFFSPRING, SELF, RTDS_NO_HOLD, RTDS_currentContext); */

/*
 * MACRO RTDS_PROCESS_KILL:
 * --------------------------
 * Kills a task representing an SDL process. An SDL task can only kill itself.
 */

#define RTDS_PROCESS_KILL \
		RTDS_ProcessKill( RTDS_currentContext );





/* ************************************************************************************ *
 *                                      FINITE STATE MACHINE
 * ************************************************************************************ */

/*
 * MACRO RTDS_PROCEDURE_CLEAN_UP:
 * ------------------------------
 * Macro called at the end of a procedure to free last message and re-organize
 * save queue if procedure contained states.
 */
#define RTDS_PROCEDURE_CLEAN_UP \
	RTDS_TransitionCleanUp( RTDS_currentContext, 0 );

/*
 * MACRO RTDS_SDL_STATE_SET:
 * -----------------------------
 * Sets the SDL state of the current process
 */

#define RTDS_SDL_STATE_SET(NEW_SDL_STATE) \
	RTDS_currentContext->sdlState = NEW_SDL_STATE; \
	RTDS_SIMULATOR_TRACE(RTDS_sdlStateSet, RTDS_currentContext->myRtosTaskId, NEW_SDL_STATE, RTDS_currentContext);

/*
 * MACRO RTDS_INFORMATION:
 * -----------------------
 * To display informal message in trace
 */
#define RTDS_INFORMATION(MESSAGE) \
    RTDS_SIMULATOR_TRACE(RTDS_information, MESSAGE, 0, RTDS_currentContext)

    
/*
 * Prototypes for debug:
 * ----------------------------------
 *
 */
RTDS_COVERAGE_PROTO;
RTDS_CRITICAL_SECTION_PROTO;
RTDS_CRITICAL_TRACE_SECTION_PROTO;
RTDS_START_SYNCHRO_PROTO;


#ifdef __cplusplus
}
#endif

#endif
